%% DESCRIPTION INPUTS

% to be defined/detup externally
% p_timeserie = column vector of daily precipitation time serie [m/d]
% et0_timeserie = column vector of daily potential evapotranspiration time serie [m/d]
% zr = depth at which the soil water content / storage and the vertical
% fluxes are calculated [m]
% s_init = soil water storage between the land surface and zr [m] 

% SSMF parameters:
% a_param = parameter for implementing the proxy of a non-unit hydraulic
% gradient, used to calculate the vertical flux
% theta_e = soil water content value between the land surface and zr [-] at which the vertical flux across zr may be
% upwards (towards the land surface)
% dt_sat = computing time step at soil saturation [hrs]
% dt_dry = computing time step at irreducible water content [hrs]

% evapotranspiration (ET) parameter. Can be replaced by any other model for ET
% c_param

% Water retention curve (wrc) parameters. We provide a matlab function to
% obtain soil water content theta from van Genuchten (vg) or Brooks and
% Corey (bc) wrc, as calculated in Hydrus sofware.
% model_wrc_str = string: 'vg' or 'bc'
% theta_r : irreducible water content between the land surface and zr [-], see Hydrus documentation
% theta_s : saturated water content between the land surface and zr [-], see Hydrus documentation
% ks : saturated hydraulic conductivity between the land surface and zr [m/day], see Hydrus documentation
% alpha : empirical parameter between the land surface and zr [1/m], see Hydrus documentation
% n : empirical parameter between the land surface and zr [-], see Hydrus documentation
% tor : tortuosity parameter between the land surface and zr [-], see Hydrus documentation

%% DESCRIPTION OUTPUTS

% storage = daily time serie of soil water storage between the land surface and zr [m/day]
% evap = daily evapoation time serie from the land surface and zr [m/day]
% flux_vertical = daily vertical flux time serie at zr [m/day]. Positive if
% downwards, negative if upwards (towards land surface)
% overlandflow = daily overland flow time serie [m/day]

%%

tic

nb_days = length(p_timeserie); % number of days of the simulation

% initialization of variables
[theta,storage,evap,flux_vertical_subroutine,flux_vertical,overlandflow,up,inf_pot_temp,inf_pot] = deal(NaN(nb_days,1));
storage(1,1) = s_init;
theta(1,1) = storage(1,1)./zr;

% calculation of daily values
for d=2:nb_days 

    theta_d = theta(d-1,1); % initial soil water content between the land surface and zr [-] for the day d

    % evap
    evap(d,1) = et0_timeserie(d)*((theta_d-theta_r)/(theta_s-theta_r))^c_param; % calculation of the evaporation for the day 1
    
    % potential water infiltration from the land surface 
    inf_pot_temp(d,1) =  p_timeserie(d,1)-evap(d,1); % potential infiltration from the land suface due to climatic conditions
    if inf_pot(d,1)<0 & theta_d>theta_e % if the potential infiltration is negative (evap>precipitation) and the soil water content is greater than the threshold
        up(d,1) = inf_pot_temp(d,1); % then there is an upwards flux across zr, equals to inf_pot
    else  % if the potential infiltration is positive (evap<precipitation) OR if it is negative and the soil water content is smaller than the threshold
        up(d,1)=0; % then there is no upwards flux across zr
    end
    inf_pot(d,1) = inf_pot_temp(d,1) - up(d,1); % resulting potential infiltration form the land surface
    
    % SSMF call
    [flux_vertical_subroutine(d,1),theta(d,1)] = ssmf_subdaily(theta_d,theta_s,ks,theta_r,zr,dt_sat,dt_dry,alpha,n,tor,inf_pot(d,1),model_wrc_str,a_param);

    % daily values
    flux_vertical(d,1) = up(d,1)+flux_vertical_subroutine(d,1); % the total vertical flux across zr is equal to sum of the upwards flux and the vertical flux calculated in the subroutine
    storage(d,1)= theta(d,1)*zr; % soil water storage between the land surface and z [m/day]
    overlandflow(d,1) = inf_pot(d,1)-flux_vertical(d,1)-(storage(d,1)-storage(d-1,1)); % overland flow [m/day]
    
    if ~isreal(storage) || storage(d,1)<0 % bug alert
        [storage,evap,flux_vertical,overlandflow] = deal(NaN(nb_days,1));
        break
    end
    
end

timelaps = toc;

figure
subplot(2,1,1)
bar(p_timeserie)
ylabel('Precipitation flux [m/d]')
xlim([1 nb_days])
subplot(2,1,2)
plot(theta)
ylabel('Soil water content [m3/m3]')
xlim([1 nb_days])

